/*
  Top 10, a racing simulator
  Copyright (C) 2003-2007  Johann Deneux
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Authors can be contacted at following electronic addresses:
  Johann Deneux: johann.deneux@gmail.com
*/

#ifndef TOP10_PHYSX_CLUTCH_STATE_HH
#define TOP10_PHYSX_CLUTCH_STATE_HH

#include "ClutchProperties.hh"

namespace top10
{
  namespace physX
  {
    //! State of a clutch.
    /*!
      A clutch has an angular velocity. No inertia, it is included in the wheel.
    */
    class ClutchState
    {
    public:
      ClutchState(const ClutchProperties*);

      void setWSpeed(double);
      double getWSpeed() const;

      //! Returns the friction coefficient.
      double getFriction() const;

      void toggle(bool b);
      bool isEngaged() const;

    private:
      //! True if the clutch is engaged.
      bool m_engaged;

      //! Angular velocity of the part connected to the wheels, in rad/s
      double m_speed;

      //! Properties of this clutch
      top10::util::Ref<const ClutchProperties> m_props;
    };
  }
}

#endif
